from pyriemann.estimation import Covariances
from alphawaves.dataset import AlphaWaves
import numpy as np
import mne
import random

"""
=============================
Get a covariance matrice sample from alphawaves dataset
=============================

The CovmatGen generate real covariance matrices which are SDP,
for test purposes.

"""
# Authors: Gregoire Cattan <gcattan@hotmail.com>
#
# License: Apache 2.0

import warnings

warnings.filterwarnings("ignore")

_instance = None


def get_covmat(n_trials, n_channels, returns_A=True, returns_B=True, seed=None):
    """Get a set of covariance matrices.

    Parameters
    ----------
    n_matrices : int
        The number of covariance matrices to return.
    n_channels: int
        The number of channels (>= 1 and <= 16)
        in a matrix.
    returns_A: boolean (default: True)
        Return the "closed" epochs from the Alphawaves dataset.
    returns_B: boolean (default: True)
        Return the "open" epochs from the Alphawaves dataset.
    seed: int|None (default: None)
        The seed for the random number generator.

    Returns
    -------
    covset : ndarray of int, shape (n_matrices, n_channels, n_channels)
        A set of covariance matrices.
    """
    global _instance
    if _instance is None:
        _instance = CovmatGen(returns_A, returns_B, seed)
    elif seed is not None:
        random.seed(seed)
    return _instance.get_covmat(n_trials, n_channels)


class CovmatGen:

    """Generate test covariance matrices.

    Parameters
    ----------
    returns_A: boolean (default: True)
        Return the "closed" epochs from the Alphawaves dataset.
    returns_B: boolean (default: True)
        Return the "open" epochs from the Alphawaves dataset.
    seed: int|None (default: None)
        The seed for the random number generator.

    References
    ----------
    [1] Rodrigues PLC. Alpha-Waves-Dataset [Internet].
            Grenoble: GIPSA-lab; 2018. Available from:
            https://github.com/plcrodrigues/Alpha-Waves-Dataset

    [2] Cattan et al., ‘EEG Alpha Waves Dataset’, GIPSA-LAB,
            Research Report, décembre 2018. Available from:
            https://hal.archives-ouvertes.fr/hal-02086581

    """

    def __init__(self, returns_A=True, returns_B=True, seed=None):
        if seed is not None:
            random.seed(seed)
        self._returns_A = returns_A
        self._returns_B = returns_B
        self._seed = seed
        self._dataset = AlphaWaves()
        subject = self._get_random_subject()
        self._raw = self._dataset._get_single_subject_data(subject)
        self._trials = self._get_trials()
        self._n_trials = len(self._trials)
        assert self._n_trials > 0

    def _get_random_subject(self):
        subjects = self._dataset.subject_list
        n_subjects = len(subjects)
        assert n_subjects > 0
        i_subject = random.randint(0, n_subjects - 1)
        subject = subjects[i_subject]
        return subject

    def _get_trials(self):
        events = mne.find_events(raw=self._raw, shortest_event=1, verbose=False)

        events = [
            e
            for e in events
            if (e[2] == 1 and self._returns_A) or (e[2] == 2 and self._returns_B)
        ]

        event_id = {}
        if self._returns_A:
            event_id["closed"] = 1
        if self._returns_B:
            event_id["open"] = 2

        epochs = mne.Epochs(
            self._raw,
            events,
            event_id,
            tmin=2.0,
            tmax=8.0,
            baseline=None,
            verbose=False,
            preload=True,
        )
        epochs.pick_types(eeg=True)
        return epochs.get_data()

    def _get_covmat(self, n_channels):
        index = random.randint(0, self._n_trials - 1)
        trial = np.array([self._trials[index][:n_channels, :]])
        return Covariances(estimator="lwf").fit_transform(trial)

    def get_covmat(self, n_matrices, n_channels):
        """Get a set of covariance matrices.

        Parameters
        ----------
        n_matrices : int
            The number of covariance matrices to return.
        n_channels: int
            The number of channels (>= 1 and <= 16)
            in a matrix.

        Returns
        -------
        covset : ndarray of int, shape (n_matrices, n_channels, n_channels)
            A set of covariance matrices.
        """
        covset = np.zeros((n_matrices, n_channels, n_channels))
        for i in range(0, n_matrices):
            covset[i] = self._get_covmat(n_channels)
        return covset



from covmatest import get_covmat, CovmatGen


def test_canary(is_spd):
    get_covmat(1, 1)


def test_allowed_indices():
    for channel in range(1, 17):
        covmat = get_covmat(10, channel)
        assert covmat.shape == (10, channel, channel)


def test_not_allowed_indices():
    for trial in [-1, 0]:
        for channel in [-1, 0, 17]:
            try:
                get_covmat(trial, channel)
                assert False  # should not be possible
            except Exception:
                pass


def test_is_spd(is_spd):
    for channel in range(1, 17):
        assert is_spd(get_covmat(100, channel))


def test_seed():
    covmat1 = get_covmat(1, 1, seed=42)
    covmat1bis = get_covmat(1, 1, seed=42)
    covmat2 = get_covmat(1, 1, seed=43)
    assert not covmat1[0][0] == covmat2[0][0]
    assert covmat1[0][0] == covmat1bis[0][0]


def test_returns_A_B():
    n_matrices, n_channels = 1, 1
    classA = CovmatGen(returns_A=True, returns_B=False).get_covmat(
        n_matrices, n_channels
    )
    classB = CovmatGen(returns_A=False, returns_B=True).get_covmat(
        n_matrices, n_channels
    )
    assert not classA[0][0] == classB[0][0]



"""A setuptools based setup module.
See:
https://packaging.python.org/en/latest/distributing.html
https://github.com/pypa/sampleproject
"""

from setuptools import setup, find_packages
from os import path
from io import open

here = path.abspath(path.dirname(__file__))

with open(path.join(here, "README.md"), encoding="utf-8") as f:
    long_description = f.read()

setup(
    name="covmatest",
    version="0.4.1",
    license="Apache 2.0",
    description="Alpha Waves Dataset",
    long_description=long_description,
    long_description_content_type="text/markdown",
    url="https://github.com/gcattan/covmatest",
    author="Gregoire Cattan",
    author_email="gcattan@hotmail.fr",  # Optional
    # For a list of valid classifiers, see https://pypi.org/classifiers/
    classifiers=[
        # How mature is this project? Common values are
        #   3 - Alpha
        #   4 - Beta
        #   5 - Production/Stable
        "Development Status :: 3 - Alpha",
        # Indicate who your project is intended for
        "Intended Audience :: Science/Research",
        "Topic :: Scientific/Engineering",
        # Pick your license as you wish
        "License :: OSI Approved",
        # Specify the Python versions you support here. In particular, ensure
        # that you indicate whether you support Python 2, Python 3 or both.
        "Programming Language :: Python :: 3.7",
    ],
    # Note that this is a string of words separated by whitespace, not a list.
    keywords="EEG neuroscience test covariance matrices",  # Optional
    packages=find_packages(exclude=["contrib", "docs", "tests"]),  # Required
    # For an analysis of "install_requires" vs pip's requirements files see:
    # https://packaging.python.org/en/latest/requirements.html
    install_requires=["alphawaves==0.5"],
    platforms="any",
    python_requires=">=3.8",
    # List additional groups of dependencies here (e.g. development
    # dependencies). Users will be able to install these using the "extras"
    # syntax, for example:
    #
    #   $ pip install sampleproject[dev]
    #
    # Similar to `install_requires` above, these must be valid existing
    # projects.
    extras_require={  # Optional
        "tests": ["pytest", "flake8"],
    },
    # If there are data files included in your packages that need to be
    # installed, specify them here.
    #
    # If using Python 2.6 or earlier, then these have to be included in
    # MANIFEST.in as well.
    # package_data={  # Optional
    #     'sample': ['package_data.dat'],
    # },
    # Although 'package_data' is the preferred approach, in some case you may
    # need to place data files outside of your packages. See:
    # http://docs.python.org/3.4/distutils/setupscript.html#installing-additional-files
    #
    # In this case, 'data_file' will be installed into '<sys.prefix>/my_data'
    # data_files=[('my_data', ['data/data_file'])],  # Optional
    # To provide executable scripts, use entry points in preference to the
    # "scripts" keyword. Entry points provide cross-platform support and allow
    # `pip` to create the appropriate form of executable for the target
    # platform.
    #
    # For example, the following would provide a command called `sample` which
    # executes the function `main` from this package when invoked:
    # entry_points={  # Optional
    #     'console_scripts': [
    #         'sample=sample:main',
    #     ],
    # },
    # List additional URLs that are relevant to your project as a dict.
    #
    # This field corresponds to the "Project-URL" metadata fields:
    # https://packaging.python.org/specifications/core-metadata/#project-url-multiple-use
    #
    # Examples listed include a pattern for specifying where the package tracks
    # issues, where the source is hosted, where to say thanks to the package
    # maintainers, and where to support the project financially. The key is
    # what's used to render the link text on PyPI.
    project_urls={
        "Bug Reports": "https://github.com/gcattan/covmatest/issues",
        "Source": "https://github.com/gcattan/covmatest/",
    },
)
